"""This module provides utility functions that are used within rfd"""
try:
    from urllib.parse import urlparse  # python 2
except ImportError:
    from urlparse import urlparse  # python 1
from bs4 import BeautifulSoup


def strip_html(text):
    return BeautifulSoup(text, "html.parser").get_text()


def is_valid_url(url):
    result = urlparse(url)
    return all([result.scheme, result.netloc, result.path])


def is_int(number):
    try:
        int(number)
        return True
    except ValueError:
        return False



import re
import json
from colorama import Fore, Style
from . import API_BASE_URL
from .scores import calculate_score, get_vote_color


class Thread:
    def __init__(self, title, dealer_name, score, url, views):
        self.dealer_name = dealer_name
        self.score = score
        self.title = title
        self.url = url
        self.views = views

    def __repr__(self):
        return f"Thread({self.title})"


class ThreadEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, Thread):
            return dict(
                dealer_name=o.dealer_name,
                score=o.score,
                title=o.title,
                url=o.url,
                views=o.views,
            )
        return json.JSONEncoder.default(self, o)


def build_web_path(slug):
    return f"{API_BASE_URL}{slug}"


def get_dealer(topic):
    dealer = None
    if topic.get("offer"):
        dealer = topic.get("offer").get("dealer_name")
    return dealer


def parse_threads(threads):
    """Parse topics list api response into digestible list.

    Arguments:
        threads {dict} -- topics response from rfd api

    Returns:
        list(dict) -- digestible list of threads
    """
    parsed_threads = []
    if threads is None:
        return []
    for topic in threads:
        parsed_threads.append(
            Thread(
                title=topic.get("title"),
                dealer_name=get_dealer(topic),
                score=calculate_score(topic),
                url=build_web_path(topic.get("web_path")),
                views=topic.get("total_views"),
            )
        )
    return parsed_threads


def sort_threads(threads, sort_by):
    """Sort threads by an attribute"""
    if sort_by is None:
        return threads
    assert sort_by in ["views", "score", "title"]
    threads = sorted(threads, key=lambda x: getattr(x, sort_by), reverse=True)
    return threads


def search_threads(threads, regex):
    """Match deal title and dealer names with regex specified."""

    regexp = re.compile(str(regex).lower())

    for deal in threads:

        if regexp.search(deal.title.lower()) or (
            deal.dealer_name and regexp.search(deal.dealer_name.lower())
        ):
            yield deal


def generate_thread_output(threads):
    for count, thread in enumerate(threads, 1):
        output = ""
        dealer = thread.dealer_name
        if dealer and dealer is not None:
            dealer = "[" + dealer + "] "
        else:
            dealer = ""
        output += (
            " "
            + str(count)
            + "."
            + get_vote_color(thread.score)
            + Fore.RESET
            + f"{dealer}{thread.title}"
            + Fore.LIGHTYELLOW_EX
            + f" ({thread.views} views)"
            + Fore.RESET
        )
        output += Fore.BLUE + f" {thread.url}"
        output += Style.RESET_ALL
        output += "\n\n"
        yield output



from __future__ import unicode_literals


import logging
import sys
import json
import click

try:
    from importlib import metadata
except ImportError:  # for Python<3.8
    import importlib_metadata as metadata
from colorama import init
from .api import get_threads, get_posts
from .threads import (
    parse_threads,
    search_threads,
    sort_threads,
    generate_thread_output,
    ThreadEncoder,
)
from .posts import generate_posts_output, PostEncoder


init()

logging.getLogger()
logging.getLogger().setLevel(logging.INFO)
logging.getLogger().addHandler(logging.StreamHandler())


def get_version():
    return "rfd v" + metadata.version("rfd")


def print_version(ctx, _, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(get_version(), nl=True)
    ctx.exit()


@click.group(invoke_without_command=True)
@click.option(
    "-v",
    "--version",
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
)
@click.pass_context
def cli(ctx):
    """CLI for https://forums.redflagdeals.com"""
    if not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


@cli.command(short_help="Display all posts in a thread.")
@click.argument("post_id")
@click.option(
    "--output", default=None, help="Defaults to custom formatting. Other options: json"
)
def posts(post_id, output):
    """Iterate all pages and display all posts in a thread.

    post_id can be a full url or post id only

    Example:

    \b
    rfd posts https://forums.redflagdeals.com/koodo-targeted-public-mobile-12-120-koodo-5gb-40-no-referrals-2173603
    """

    try:
        if output == "json":
            click.echo_via_pager(
                json.dumps(
                    get_posts(post=post_id),
                    cls=PostEncoder,
                    indent=2,
                    sort_keys=True,
                )
            )
        else:
            click.echo_via_pager(generate_posts_output(get_posts(post=post_id)))

    except ValueError:
        click.echo("Invalid post id.")
        sys.exit(1)
    except AttributeError as err:
        click.echo("The RFD API did not return the expected data. %s", err)
        sys.exit(1)


@cli.command(short_help="Displays threads in the forum. Defaults to hot deals.")
@click.option("--forum-id", default=9, help="The forum id number")
@click.option("--pages", default=1, help="Number of pages to show. Defaults to 1.")
@click.option("--sort-by", default=None, help="Sort threads by")
@click.option(
    "--output", default=None, help="Defaults to custom formatting. Other options: json"
)
def threads(forum_id, pages, sort_by, output):
    """Display threads in the specified forum id. Defaults to 9 (hot deals).

    Popular forum ids:

    \b
    9 \t hot deals
    14 \t computer and electronics
    15 \t offtopic
    17 \t entertainment
    18 \t food and drink
    40 \t automotive
    53 \t home and garden
    67 \t fashion and apparel
    74 \t shopping discussion
    88 \t cell phones
    """
    _threads = sort_threads(
        parse_threads(get_threads(forum_id, pages)), sort_by=sort_by
    )
    if output == "json":

        click.echo_via_pager(
            json.dumps(
                sort_threads(_threads, sort_by=sort_by),
                cls=ThreadEncoder,
                indent=2,
                sort_keys=True,
            )
        )
    else:
        click.echo_via_pager(generate_thread_output(_threads))


@cli.command(short_help="Search deals based on a regular expression.")
@click.option("--pages", default=5, help="Number of pages to search.")
@click.option(
    "--forum-id", default=9, help="The forum id number. Defaults to 9 (hot deals)."
)
@click.option("--sort-by", default=None, help="Sort threads by")
@click.option(
    "--output", default=None, help="Defaults to custom formatting. Other options: json"
)
@click.argument("regex")
def search(pages, forum_id, sort_by, output, regex):
    """Search deals based on regex.

    Popular forum ids:

    \b
    9 \t hot deals
    14 \t computer and electronics
    15 \t offtopic
    17 \t entertainment
    18 \t food and drink
    40 \t automotive
    53 \t home and garden
    67 \t fashion and apparel
    74 \t shopping discussion
    88 \t cell phones
    """

    matched_threads = []

    _threads = parse_threads(get_threads(forum_id, pages=pages))
    for thread in search_threads(threads=_threads, regex=regex):
        matched_threads.append(thread)

    if output == "json":
        click.echo_via_pager(
            json.dumps(
                sort_threads(matched_threads, sort_by=sort_by),
                indent=2,
                sort_keys=True,
                cls=ThreadEncoder,
            )
        )
    else:
        click.echo_via_pager(
            generate_thread_output(sort_threads(matched_threads, sort_by=sort_by))
        )



"""RFD API."""

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError
import logging
import requests
from . import API_BASE_URL
from .posts import Post
from .scores import calculate_score
from .utils import is_int, strip_html, is_valid_url


def extract_post_id(url):
    return url.split("/")[3].split("-")[-1]


def get_safe_per_page(limit):
    """Ensure that per page limit is between 5-40"""
    if limit < 5:
        return 5
    if limit > 40:
        return 40
    return limit


def create_user_map(users):
    """Create a map of user ids to usernames."""
    m = {}
    for user in users:
        m[user.get("user_id")] = user.get("username")
    return m


def get_threads(forum_id, pages):
    """Get threads from rfd api

    Arguments:
        forum_id {int} -- forum id
        pages {int} -- the number of pages of threads to collect

    Returns:
        dict -- api response
    """
    threads = []
    try:
        for page in range(1, pages + 1):
            response = requests.get(
                f"{API_BASE_URL}/api/topics?forum_id={forum_id}&per_page=40&page={page}"
            )
            if response.status_code != 200:
                raise Exception(
                    f"When collecting threads, received a status code: {response.status_code}"
                )
            threads += response.json().get("topics")
    except JSONDecodeError as err:
        logging.error("Unable to decode threads. %s", err)
    return threads


def get_posts(post):
    """Retrieve posts from a thread.

    Args:
        post (str): either full url or postid

    Returns:
        list(Post): Posts
    """
    if is_valid_url(post):
        post_id = extract_post_id(post)
    elif is_int(post):
        post_id = post
    else:
        raise ValueError()

    response = requests.get(
        f"{API_BASE_URL}/api/topics/{post_id}/posts?per_page=40&page=1"
    )

    total_pages = response.json().get("pager").get("total_pages")

    posts = []

    for page in range(0, total_pages + 1):
        response = requests.get(
            f"{API_BASE_URL}/api/topics/{post_id}/posts?per_page=40&page={page}"
        )
        users = create_user_map(response.json().get("users"))

        current_posts = response.json().get("posts")

        for _post in current_posts:
            # Sometimes votes is null
            if _post.get("votes") is not None:
                calculated_score = calculate_score(_post)
            else:
                calculated_score = 0
            posts.append(
                Post(
                    body=strip_html(_post.get("body")),
                    score=calculated_score,
                    user=users[_post.get("author_id")],
                )
            )

    return posts



# pylint: disable=old-style-class
import os
import json
from colorama import Fore, Style
from .scores import get_vote_color


class Post:
    def __init__(self, body, score, user):
        self.body = body
        self.score = score
        self.user = user


class PostEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, Post):
            return dict(
                body=o.body,
                score=o.score,
                user=o.user,
            )
        return json.JSONEncoder.default(self, o)


def get_terminal_width():
    _, columns = os.popen("stty size", "r").read().split()
    return int(columns)


def generate_posts_output(posts):
    output = ""
    output += "-" * get_terminal_width()
    for post in posts:
        output += (
            " -"
            + get_vote_color(post.score)
            + Fore.RESET
            + post.body
            + Fore.YELLOW
            + f" ({post.user})"
        )
        output += Style.RESET_ALL
        output += "\n"
        output += "-" * get_terminal_width()
        output += "\n"
    return output



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rfd']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.10.1',
 'click>=7.0',
 'colorama>=0.4.3',
 'requests>=2.22.0',
 'soupsieve<3.0']

entry_points = \
{'console_scripts': ['rfd = rfd.__main__:cli']}

setup_kwargs = {
    'name': 'rfd',
    'version': '0.9.0',
    'description': 'view RedFlagDeals.com from the command line',
    'long_description': '# RFD\n\n[![PyPI version](https://badge.fury.io/py/rfd.svg)](https://badge.fury.io/py/rfd)\n[![Dependabot](https://badgen.net/badge/Dependabot/enabled/green?icon=dependabot)](https://dependabot.com/)\n[![Downloads](https://pepy.tech/badge/rfd)](https://pepy.tech/project/rfd)\n\n<!-- BEGIN mktoc -->\n- [Description](#description)\n- [Motivation](#motivation)\n- [Installation](#installation)\n- [Usage](#usage)\n  - [View Hot Deals](#view-hot-deals)\n  - [View and Sort Hot Deals](#view-and-sort-hot-deals)\n  - [Search](#search)\n    - [Regex](#regex)\n  - [View Posts](#view-posts)\n  - [JSON Output](#json-output)\n- [Shell Completion](#shell-completion)\n  - [bash](#bash)\n  - [zsh](#zsh)\n<!-- END mktoc -->\n\n## Description\n\nThis is a CLI utility that allows you to view [RedFlagDeals.com](https://forums.redflagdeals.com) on the command line.\n\n![screenshot](https://user-images.githubusercontent.com/4519234/85969861-e10a4100-b996-11ea-9a31-6203322c60ee.png)\n\n## Motivation\n\nIt is often faster to use a CLI than to load up a web page and navigate web elements. This tool can search for deals and sort them based on score and views. It is also able to load entire threads (without pagination) for additional analysis.\n\n## Installation\n\n### pip\n\n```sh\npip3 install --user rfd\n```\n\nThis can also be installed with [pipx](https://github.com/pypa/pipx).\n\n### brew\n\nIf you have [brew](https://brew.sh):\n\n```sh\nbrew install davegallant/public/rfd\n```\n\n## Usage\n\nAll commands open up in a [terminal pager](https://en.wikipedia.org/wiki/Terminal_pager).\n\n```sh\nUsage: rfd [OPTIONS] COMMAND [ARGS]...\n\n  CLI for https://forums.redflagdeals.com\n\nOptions:\n  -v, --version\n  --help         Show this message and exit.\n\nCommands:\n  posts    Display all posts in a thread.\n  search   Search deals based on a regular expression.\n  threads  Displays threads in the forum. Defaults to hot deals.\n```\n\n### View Hot Deals\n\nTo view the threads on most popular sub-forum:\n\n```sh\nrfd threads\n```\n\n### View and Sort Hot Deals\n\n```sh\nrfd threads --sort-by score\n```\n\nTo view and sort multiple pages, use `--pages`:\n\n```sh\nrfd threads --sort-by views --pages 10\n```\n\n### Search\n\n```sh\nrfd search \'pizza\'\n```\n\n#### Regex\n\nRegular expressions can be used for search.\n\n```sh\nrfd search \'(coffee|starbucks)\' --pages 10 --sort-by views\n```\n\n### View Posts\n\nIt\'s possible to view an entire post and all comments by running:\n\n```sh\nrfd posts https://forums.redflagdeals.com/kobo-vs-kindle-2396227/\n```\n\nThis allows for easy grepping and searching for desired expressions.\n\n### JSON Output\n\nAll commands support JSON output.\n\nFor example:\n\n```sh\nrfd threads --output json\n```\n\n## Shell Completion\n\nShell completion can be enabled if using `bash` or `zsh`.\n\n### bash\n\n```sh\necho \'eval "$(_RFD_COMPLETE=source rfd)"\' >> ~/.profile\n```\n\n### zsh\n\n```sh\necho \'eval "$(_RFD_COMPLETE=source_zsh rfd)"\' >> ~/.zshrc\n```\n',
    'author': 'Dave Gallant',
    'author_email': 'davegallant@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
